#pragma  once

#define TIXML_USE_STL
#include "tinyxml.h"

// [:2008-09-09] stricmp -> _stricmp   

class XMLUtil
{
public:
	static LPSTR  Encode_utf8(LPCWSTR wStr)     // UNICODE -> UTF8
	{
		int size = WideCharToMultiByte(CP_UTF8, 0, wStr, -1, NULL, 0, NULL, NULL);         
		LPSTR utf8Str = new char[size+1];    
		WideCharToMultiByte(CP_UTF8, 0, wStr, -1, utf8Str, size, 0, 0);    
		utf8Str[size] = 0;
		return utf8Str;
	}

	static LPWSTR Decode_utf8(LPCSTR utf8Str) // UTF8 -> UNICODE
	{
		int size = MultiByteToWideChar(CP_UTF8, 0,  utf8Str, -1, NULL, 0);
		LPWSTR wStr = new WCHAR[size+1];
		MultiByteToWideChar(CP_UTF8, 0,  utf8Str, -1, wStr, size);
		wStr[size] = 0;
		return wStr;
	}
	static BOOL GetValue(TiXmlElement* pParentNode, LPCSTR pszName, BOOL& bValue)
	{
		TCHAR szValue[32] = {0};
		if (GetValue(pParentNode, pszName, szValue, 32))
		{
			bValue = (_tcsicmp(szValue, _T("true")) == 0);
			return TRUE;
		}
		return FALSE;
	}
	static BOOL GetValue(TiXmlElement* pParentNode, LPCSTR pszName, LONG& lValue)
	{
		TiXmlElement* pValueNode = pParentNode->FirstChildElement(pszName);
		if (pValueNode)
		{
			TiXmlNode* pChildNode = pValueNode->FirstChild();
			if (pChildNode)
			{
				lValue = atol(pChildNode->Value());
				return TRUE;
			}
		}
		return FALSE;
	}

	static BOOL GetValue(TiXmlElement* pParentNode, LPCSTR pszName, TCHAR* pszValue, INT nMax)
	{
		pszValue[0] = _T('\0');	
		TiXmlElement* pValueNode = pParentNode->FirstChildElement(pszName);
		if (pValueNode)
		{
			return GetValue(pValueNode, pszValue, nMax);			
		}
		return FALSE;
	}
	static BOOL GetValue(TiXmlElement* pValueNode, TCHAR* pszValue, INT nMax)
	{
		TiXmlNode* pChildNode = pValueNode->FirstChild();
		while (pChildNode)
		{	
			if (pChildNode->Type() == TiXmlNode::COMMENT)
			{
				pChildNode = pChildNode->NextSibling();
				continue;
			}
			return GetChildValue(pChildNode, pszValue, nMax);
		}
		return FALSE;
	}
	static BOOL GetChildValue(TiXmlNode* pChildNode, TCHAR* pszValue, INT nMax)
	{
		ZeroMemory(pszValue, nMax);
		USES_CONVERSION;	
		if (pChildNode->Type() == TiXmlNode::COMMENT) return FALSE;
		if (pChildNode->Type() == TiXmlNode::TEXT)
		{
			TiXmlText* pText = pChildNode->ToText();
			if( pText)
			{
				if (pText->CDATA())
				{
					_tcsncpy(pszValue, A2CT(pChildNode->Value()), nMax);
					//return TRUE;
				}
			}
		}
		TiXmlNode* pDocChildNode = pChildNode->GetDocument()->FirstChild();
		if (pDocChildNode)
		{
			if (pDocChildNode->Type() == TiXmlNode::DECLARATION)
			{
				TiXmlDeclaration* pDeclarationNode = pDocChildNode->ToDeclaration();
				if (pDeclarationNode)
				{
					LPCSTR pszcEncoding = pDeclarationNode->Encoding();
					if (_stricmp(pszcEncoding, "UTF-8") == 0 ||
						_stricmp(pszcEncoding, "UTF8") == 0)
					{
						wchar_t* pDecodeValue = XMLUtil::Decode_utf8(pChildNode->Value());
						_tcsncpy(pszValue, W2CT(pDecodeValue), nMax);
						delete[] pDecodeValue;
					}
				}
			}
		}
		if (_tcslen(pszValue) == 0)
		{
			_tcsncpy(pszValue, A2CT(pChildNode->Value()), nMax);
		}
		return TRUE;
	}
	static BOOL GetAttrValue(TiXmlElement* pParentNode, LPCSTR pszName, LONG& lValue)
	{
		int iValue = 0;
		LPCSTR pValue = pParentNode->Attribute(pszName, &iValue);
		if (pValue)
		{
			lValue = atol(pValue);
			return TRUE;
		}	
		return FALSE;
	}

	static BOOL GetValueAsAttr(TiXmlElement* pParentNode, LPCSTR pszName, TCHAR* pszValue, INT nMax)
	{
		CStringA strElement = pParentNode->Value();
		strElement.Append(".");
		strElement.Append(pszName);

		return GetValue(pParentNode, strElement, pszValue, nMax);
	}
	static BOOL GetValueAsSubAttr(TiXmlElement* pParentNode, LPCSTR pszName, TCHAR* pszValue, INT nMax)
	{
		CStringA strElement = pParentNode->Value(); //ex : simpletext.font
		int nFindIndex = strElement.ReverseFind(_T('.'));
		strElement = strElement.Right(strElement.GetLength() - nFindIndex - 1); //font
		strElement.Append("."); //font.
		strElement.Append(pszName); //font.[pszName]

		return GetValue(pParentNode, strElement, pszValue, nMax);
	}
	static BOOL GetValueAsAttr(TiXmlElement* pParentNode, LPCSTR pszName, LONG& lValue)
	{
		CStringA strElement = pParentNode->Value();
		strElement.Append(".");
		strElement.Append(pszName);

		return GetValue(pParentNode, strElement, lValue);
	}
	static BOOL GetValueAsSubAttr(TiXmlElement* pParentNode, LPCSTR pszName, LONG& lValue)
	{
		CStringA strElement = pParentNode->Value(); //ex : simpletext.font
		int nFindIndex = strElement.ReverseFind('.');
		strElement = strElement.Right(strElement.GetLength() - nFindIndex - 1); //font
		strElement.Append("."); //font.
		strElement.Append(pszName); //font.[pszName]

		return GetValue(pParentNode, strElement, lValue);
	}
	static BOOL GetAttrValue(TiXmlElement* pParentNode, LPCSTR pszName, TCHAR* pszValue, INT nMax)
	{
		pszValue[0] = _T('\0');

		LPCSTR pValue = pParentNode->Attribute(pszName);
		if (pValue)
		{
			USES_CONVERSION;
			TiXmlNode* pDocChildNode = pParentNode->GetDocument()->FirstChild();
			if (pDocChildNode)
			{
				if (pDocChildNode->Type() == TiXmlNode::DECLARATION)
				{
					TiXmlDeclaration* pDeclarationNode = pDocChildNode->ToDeclaration();
					if (pDeclarationNode)
					{
						LPCSTR pszcEncoding = pDeclarationNode->Encoding();
						if (_stricmp(pszcEncoding, "UTF-8") == 0 ||
							_stricmp(pszcEncoding, "UTF8") == 0)
						{
							wchar_t* pDecodeValue = Decode_utf8(pValue);
							_tcsncpy(pszValue, W2CT(pDecodeValue), nMax);
							delete[] pDecodeValue;
						}
					}
				}
			}
			if (_tcslen(pszValue) == 0)
			{
				_tcsncpy(pszValue, A2CT(pValue), nMax);
			}
			return TRUE;
		}
		return FALSE;
	}
	static COLORREF ColorFromString(LPCTSTR lpszColor)
	{
		COLORREF rgb = RGB(0,0,0);		// initialize to black
		BYTE r = 0;
		BYTE g = 0;
		BYTE b = 0;
		const TCHAR *cp;

		if ((cp = _tcschr(lpszColor, _T(','))) != NULL)
		{
			// "255,0,0"
			r = (BYTE) _ttoi(lpszColor);
			cp++;
			g = (BYTE) _ttoi(cp);
			cp = _tcschr(cp, _T(','));
			if (cp)
			{
				cp++;
				b = (BYTE) _ttoi(cp);
			}
			rgb = RGB(r,g,b);
		}
		else if ((cp = _tcschr(lpszColor, _T('#'))) != NULL)
		{
			// "#0000FF"
			if (_tcslen(lpszColor) == 7)
			{
				TCHAR s[3];
				cp++;
				s[0] = *cp++;
				s[1] = *cp++;
				s[2] = _T('\0');
				r = (BYTE)_tcstoul(s, NULL, 16);
				s[0] = *cp++;
				s[1] = *cp++;
				g = (BYTE)_tcstoul(s, NULL, 16);
				s[0] = *cp++;
				s[1] = *cp++;
				b = (BYTE)_tcstoul(s, NULL, 16);
				rgb = RGB(r,g,b);
			}
		}
		return rgb;
	}

	static void WriteTag(CString& strStream, LPCTSTR pszcTag, BOOL bEnd=FALSE)
	{
		strStream.AppendChar(_T('<'));
		if (bEnd)
		{
			strStream.AppendChar(_T('/'));
		}
		strStream.Append(pszcTag);
		strStream.AppendChar(_T('>'));
	}
	static void WriteElementValue(CString& strStream, LPCTSTR pszcElement, LPCTSTR pszcValue)
	{
		if (_tcslen(pszcValue) <= 0)
		{
			ATLTRACE(_T("XMLUtil::WriteElementValue #1 pszcElement(%s)\n"), pszcElement);
		}
		WriteTag(strStream, pszcElement);
		strStream.Append(pszcValue);
		WriteTag(strStream, pszcElement, TRUE);
	}
	static void WriteElementValue(CString& strStream, LPCTSTR pszcElement, LONG lValue)
	{
		CString strValue;
		strValue.Format(_T("%d"), lValue);
		WriteElementValue(strStream, pszcElement, strValue);
	}
	static void WriteElementValue(CString& strStream, LPCTSTR pszcElement, LONGLONG lValue)
	{
		CString strValue;
		strValue.Format(_T("%l64"), lValue);
		WriteElementValue(strStream, pszcElement, strValue);
	}
	static void WriteElementValue(CString& strStream, LPCTSTR pszcElement, BOOL bValue)
	{
		CString strValue;
		strValue.Format(_T("%s"), bValue ? _T("true") : _T("false"));
		WriteElementValue(strStream, pszcElement, strValue);
	}
	static void WriteElementValue(CString& strStream, LPCTSTR pszcElement, COLORREF clrValue)
	{
		CString strValue = StringFromColor(clrValue);
		WriteElementValue(strStream, pszcElement, strValue);
	}
	static CString StringFromColor( COLORREF clrColor )
	{
		CString strColor;
		strColor.Format(_T("#%02X%02X%02X"), GetRValue( clrColor ), GetGValue( clrColor ), GetBValue( clrColor ) );
		return strColor;
	}
};